

#############################################################################
### Re-constructing (predicted) wild type trait values from infected data ###
#############################################################################

predict.traitvalues = function(x=NULL, emm.file="morpho_EMM_df.csv", pars.file="morpho_TraitDistrParams_df.csv", sep=";"){
  
  # x -- new data table of infected specimens: three columns should be "species", "trait", and "trait.value"
      # species -- full species name (e.g. "Temnothorax sordidulus")
      # trait -- trait code, such as: CL, POC, CW, FR, SL, ML, PEH, PEL, NOH, NOL, PPH, PPL, SPST, SPL, MW, PEW, PPW, SPBA, SPWI, SPTI
      # trait.value -- trait values (i.e. measurements) in the 'millimeter scale
  # emm.file -- name of EMM dataframe file, along with directory path (e.g. "C:/Folder1/morpho_EMM_df.csv" )
  # pars.file -- name of datafile containing maximum likelihood mean and SD parameters for traits (e.g. "C:/Folder1/morpho_TraitDistrParams_df.csv" )
  # sep -- separator character in the datafile
  
  require("utils")
  
  emm.df = read.csv(file = emm.file, sep = sep)
  pars.df = read.csv(file = pars.file, sep = sep)
  
  shifted.df = x
  for(i in 1:nrow(shifted.df)){
    
    sp = shifted.df$species[i]
    tr = shifted.df$trait[i]
    tr.val = x$trait.value[i]
    
    ml.mean.inf = pars.df$mean[pars.df$phenotype=="infected" & pars.df$species==sp & pars.df$trait==tr]
    ml.sd.inf = pars.df$SD[pars.df$phenotype=="infected" & pars.df$species==sp & pars.df$trait==tr]
    ml.sd.wild = pars.df$SD[pars.df$phenotype=="wild" & pars.df$species==sp & pars.df$trait==tr]
    d.inf = (tr.val - ml.mean.inf)/ml.sd.inf
    s.pv = (-1)*emm.df$infected.dif[emm.df$species==sp & emm.df$trait==tr]
    trait.inf.shifted = d.inf*ml.sd.wild + ml.mean.inf + s.pv
    
    shifted.df$trait.value[i] = trait.inf.shifted
    
  }
  
  return(shifted.df)
  
}

# Example
x = data.frame(
  species = c("Temnothorax sordidulus", "Temnothorax nylanderi", "Temnothorax unifasciatus"),
  trait = rep("CL", 3),
  trait.value = c(0.68500, 0.57500, 0.61750)
)
predict.traitvalues(x = x, emm.file = "C:/morpho_EMM_df.csv", pars.file = "C:/morpho_TraitDistrParams_df.csv", sep = ";")

